use crate::commands::clone::UrlProps;
use reqwest::{header::HeaderMap, Method, Url};
use reqwest::{Client, ClientBuilder, RequestBuilder};

const USER_AGENT: &str = "Nextsync";

pub struct ClientConfig {
    client: Option<ClientBuilder>,
}

impl ClientConfig {
    pub fn new() -> Self {
        ClientConfig {
            client: Some(Client::builder().user_agent(USER_AGENT)),
        }
    }

    pub fn default_headers(mut self, headers: HeaderMap) -> Self {
        self.client = Some(
            self.client
                .take()
                .expect("Client was already built")
                .default_headers(headers),
        );
        self
    }

    pub fn build(&mut self) -> Client {
        self.client
            .take()
            .expect("Cannot build the client twice")
            .build()
            .unwrap()
    }
}

#[derive(Clone)]
pub struct Service {
    /// http[s]://host.xz/remote.php/dav/files
    url_base: String,
}

impl From<&UrlProps<'_>> for Service {
    fn from(url_props: &UrlProps) -> Self {
        let mut url_base = if url_props.is_secure {
            String::from("https://")
        } else {
            String::from("http://")
        };
        url_base.push_str(url_props.domain);
        url_base.push_str("/remote.php/dav/files");

        Service { url_base }
    }
}

impl Service {
    fn authenticate(&self, request: RequestBuilder) -> RequestBuilder {
        request
            .bearer_auth("rK5ud2NmrR8p586Th7v272HRgUcZcEKIEluOGjzQQRj7gWMMAISFTiJcFnnmnNiu2VVlENks")
    }
}

pub struct Request<'a> {
    service: &'a Service,
    client: ClientConfig,
    method: Option<Method>,
    url: Option<String>,
    headers: HeaderMap,
    body: Option<String>,
}

impl<'a> Request<'a> {
    pub fn new(service: &'a Service) -> Self {
        Request {
            service,
            client: ClientConfig::new(),
            method: None,
            url: None,
            headers: HeaderMap::new(),
            body: None,
        }
    }

    pub fn propfind(&mut self, url: String) -> &mut Self {
        self.method = Some(Method::from_bytes(b"PROPFIND").expect("Cannot be invalid"));
        self.url = Some(url);
        self
    }

    pub async fn send(&mut self) -> Result<reqwest::Response, reqwest::Error> {
        self.service
            .authenticate(self.client.build().request(
                self.method.clone().expect("Method must be set"),
                {
                    let mut url = self.service.url_base.clone();
                    url.push_str(&self.url.clone().expect("An url must be set"));
                    url
                },
            ))
            .send()
            .await
        // let mut url = self
        //     .config
        //     .clone()
        //     .expect("A config must be provided to service")
        //     .get_nsconfig()
        //     .get_remote("origin")
        //     .url
        //     .expect("An url must be set on the remote");
        // url.push_str(&self.url.clone().unwrap());

        // self.client
        //     .build()
        //     .request(self.method.clone().expect("Method must be set"), url)
        //     .send()
        //     .await
    }
}
